package fxHarakka.luokat;

import java.util.ArrayList;

/**---------------Projektien (monikko) luokka--------------------------------
 * |                                                    | -Projekti         |
 * | - lisää ja poistaa projekteja                      |                   |
 * |                                                    |                   |
 * | - Huolehtii listan pituuden riittävyydestä jos     |                   |
 * |    täyttyy                                         |                   |
 * |                                                    |                   |
 * | - Lukee ja kirjoittaa projektit.dat tiedostoon     |                   |
 * |                                                    |                   |
 * | - Etsii ja lajittelee (avustajana projekti luokka) |                   |
 */
public class Projektit {
    //private int maxMaara; //tätä ei näillänäkymin tarvitse johtuen tietorakenteesta
                            //--> ei tarvitse itse huolehtia kasvatuksesta
    private String kokoNimi = ""; //projektit nimi
    private String tied = "";     //tiedostosijainti josta luetaan ja kirjoitetaa
    private ArrayList<Projekti> projektiAlkiot = new ArrayList<>(); //varaa autom. oikean määrän alkioita

    //arraylist on nyt staattinen mutta oliot sisällä ei eli pitäis olla ok?? jos tulee ongelmia niin
    //pitää mennä Projekti luokkaan ja tehdä lisaaProjekti ei-staattiseksi ja alla lisaaProjekti myös


    //-----------------------MUODOSTAJAT - PROJEKTIT----------------------------

    /**
     *
     *
     * @param kokoNimi  projektikokoelman nimi
     * @param tied      tiedostosijainti josta luetaan ja talletetaan
     */
    public Projektit(String kokoNimi, String tied) {
        this.kokoNimi = kokoNimi;
        this.tied = tied;
    }

    /**
     * Oletusmuodostaja. voidaan käyttää ylempää jos halutaan muokata ominaisuuksia kuten nimeä esim tilanteessa
     * jossa halutaan jatkossa lisätä uusi "istunto".
     *
     *
     * oletuksena luo 10 tyhjää alkioita
     */
    public Projektit() {
        this.kokoNimi = "Harakka";
        this.tied = "projektit.dat";
    }



    //-----------------------METODIT - PROJEKTIT----------------------------


    /**
     * Lisää projektin tietorakenteeseen
     * @param projekti lisättävä projekti
     * @example <pre name="test">
     *
     * #import fxHarakka.luokat.Harakka.*;
     * #import fxHarakka.luokat.Projektit.*;
     * #import java.io.ByteArrayOutputStream;
     * #import java.io.PrintStream;
     *
     * Harakka harakka = new Harakka();
     * PrintStream alkuperainen = System.out;
     * ByteArrayOutputStream ulosByte = new ByteArrayOutputStream();
     * System.setOut(new PrintStream(ulosByte));
     *
     * Projekti projektiT1 = new Projekti("Projekti1", "Fysiikka");
     * harakka.projektit.lisaaProjekti(projektiT1);
     *
     * String strg = ulosByte.toString();
     * strg = strg.trim();
     *
     * strg === "projekti lisätty! Projekti1 | Fysiikka | 1 | 0 | 0";
     * System.setOut(alkuperainen);
     * </pre>
     */
    public void lisaaProjekti(Projekti projekti){
        projekti.rekisteroiProjekti(); //rekisteröidään ennen lisäämistä --> id päivittyy
        projektiAlkiot.add(projekti);
        System.out.println("projekti lisätty! " + projekti);
    }


    /** Poistaa valitun projektin tietokannasta
     *
     * @param poistettavanNimi etsittävän projekin nimi
     */
    public void poistaProjekti(String poistettavanNimi) {
        //etsitään poistettavan alkion indeksin tietorakenteesta
        int poistettavanIndeksi = etsiNimi(poistettavanNimi);
        if (poistettavanIndeksi == -1){
            System.out.println("poistettavaa projektia ei löytynyt; tarkista nimi");
            return; //poistutaan aliohjelmasta jos ei löydy
        }
            String poistoViesti = projektiAlkiot.get(poistettavanIndeksi).toString();
            projektiAlkiot.remove(poistettavanIndeksi);
            System.out.println("Poistettu projekti! : " + poistoViesti);
    }


    /**etsii projektin nimen perusteella tietorakenteesta sen indeksin ja palauttaa indeksin
     *
     * @return Nimeltä etsittävän projektin indeksi tai -1 jos ei löytynyt
     *
     */
    public int etsiNimi(String etsittavanNimi) {
        for (Projekti p: projektiAlkiot) { //toteuttaa iteraattorin --> foreach käy läpi ja etsii
            if (p.getNimi().contentEquals(etsittavanNimi)) return projektiAlkiot.indexOf(p);
        }
        return -1;
    }






}
